#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <omp.h>

float sumf(float *x, size_t n)
{
    float sum = 0.f;

    #pragma omp parallel for reduction(+:sum)
    for (size_t i = 0; i < n; i++) {
        sum += x[i];
    }
    return sum;
}

double sum(float *x, size_t n)
{
    double sum = 0.;
    for (size_t i = 0; i < n; i++) {
        sum += (double)x[i];
    }
    return sum;
}

float rel_error(float true_sum, float approx_sum)
{
    return fabsf((true_sum - approx_sum) / true_sum);
}

int main(int argc, char **argv)
{
    if (argc != 2) {
        printf("Usage: %s N\n"
               "\tN: number of floats in your vector\n", argv[0]);
        return EXIT_FAILURE;
    }

    size_t n = atol(argv[1]);

    /* Uniform in [0, 1) */
    float *x = malloc(n * sizeof(float));
    srand(time(NULL));
//    for (size_t i = 0; i < n; i++) {
//        int random = rand();
//        unsigned int randomu = 2 * *((unsigned int *)&random);
//        x[i] = (randomu >> 8) * 0x1p-24;
    float sum1 = 0.f;
    float sum2 = 0.f;
    float sum3 = 0.f;
    float sum4 = 0.f;
    for (size_t i = 0; i < n; i += 4) {
        sum1 += 0.5;
        sum2 += 1.0;
        sum3 += -1.5;
        sum4 += 0.0;
    }

    printf("%d %d %d %d\n", (int)sum1, (int)sum2, (int)sum3, (int)sum4);

    float approx_sum = sumf(x, n);
    double true_sum  = sum(x, n);
    printf("Relative error %e\n", approx_sum);//rel_error((float)true_sum, approx_sum));

    free(x);

    return EXIT_SUCCESS;
}
